package com.s3ai.services;

import com.s3ai.TestUtils;
import com.s3ai.entities.Cinema;
import com.s3ai.entities.Ticket;
import com.s3ai.entities.User;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import javax.persistence.NoResultException;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

class TicketServiceTest {

    private TicketService ticketServiceUnderTest;

    @BeforeEach
    void setUp() throws IOException {
        ticketServiceUnderTest = TicketService.getInstance();
        TestUtils.prepareTickets();
    }

    @Test
    void testGetAllTickets() {
        // Setup
        final int expectedResult = 100;

        // Run the test
        final List<Ticket> result = ticketServiceUnderTest.getAllTickets();

        // Verify the results
        assertEquals(100, result.size());
    }

    @Test
    void testGetTicketById() {
        // Setup
        final Ticket expectedResult = TestUtils.getRandomElement(ticketServiceUnderTest.getAllTickets());

        // Run the test
        final Ticket result = ticketServiceUnderTest.getTicketById(expectedResult.getId());

        // Verify the results
        assertEquals(expectedResult, result);
    }

    @Test
    void testSaveTicket() {
        // Setup
        final Ticket ticket = TestUtils.getRandomTicket();

        // Run the test
        ticketServiceUnderTest.saveTicket(ticket);

        // Verify the results
        assertEquals(ticket, ticketServiceUnderTest.getTicketById(ticket.getId()));

    }

    @Test
    void testUpdateTicket() {
        // Setup
        final Ticket ticket = TestUtils.getRandomElement(ticketServiceUnderTest.getAllTickets());
        ticket.setMovie("TESTING");
        // Run the test
        ticketServiceUnderTest.updateTicket(ticket);


        // Verify the results
        assertEquals("TESTING", ticketServiceUnderTest.getTicketById(ticket.getId()).getMovie());
    }

    @Test
    void testDeleteTicket() {
        // Setup
        final Ticket ticket = TestUtils.getRandomElement(ticketServiceUnderTest.getAllTickets());
        UUID id = ticket.getId();
        // Run the test
        ticketServiceUnderTest.deleteTicket(ticket);

        // Verify the results
        assertThrows(NoResultException.class, () -> ticketServiceUnderTest.getTicketById(id));
    }

    @Test
    void testFillTicketFields() {
        // Setup
        final Ticket ticket = new Ticket();

        final User user = new User();

        final Cinema cinema = new Cinema();

        // Run the test
        ticketServiceUnderTest.fillTicketFields(ticket, user, cinema, "movie");

        // Verify the results
        assertEquals(cinema, ticket.getCinema());
        assertEquals(user, ticket.getUser());
        assertEquals("movie", ticket.getMovie());
    }

    @Test
    void testGetInstance() {
        // Setup
        final TicketService expectedResult = TicketService.getInstance();

        // Run the test
        final TicketService result = TicketService.getInstance();

        // Verify the results
        assertEquals(expectedResult, result);
    }

}
